import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:intl/intl.dart';

void main() => runApp(MyApp());

class MyApp extends StatelessWidget {
  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  MyHomePage({Key key, this.title}) : super(key: key);

  final String title;

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _DateTimeItem {
  DateTime dateTime;
  bool canSelect = true;
  bool isSelected = true;
}

class _MyHomePageState extends State<MyHomePage> {
  List<_DateTimeItem> _dateTimeList = List<_DateTimeItem>();

  static DateFormat _format = DateFormat("y-MM-d HH:00");
  DateTime _startDateTime = _format.parse(_format.format(DateTime.now().add(Duration(hours: 1))));
  DateTime _dateTimeSelected = _format.parse('2019-10-27 02:00');
  int _dateTimeIndex;

  @override
  void initState() {
    // TODO: implement initState
    _fillDateTimeList(_startDateTime, _dateTimeSelected);
    super.initState();
  }

  void _incrementCounter() {
    setState(() {
      _fillDateTimeList(_startDateTime, _dateTimeSelected, preHours: 150);
    });
  }

  void _fillDateTimeList(DateTime startDateTime, DateTime dateTimeSelected, {int preHours = 13}) {
    _dateTimeList.clear();
    _DateTimeItem _dateTimeItem;
    int _length = startDateTime.difference(dateTimeSelected.subtract(Duration(hours: preHours))).inHours;
    for (int i1 = 0; i1 < _length; i1++) {
      _dateTimeItem = _DateTimeItem();
      _dateTimeItem.dateTime = startDateTime.subtract(Duration(hours: i1));
      _dateTimeItem.canSelect = true;//_dateTimeItem.dateTime.hour != 0;
      if (dateTimeSelected.isAtSameMomentAs(_dateTimeItem.dateTime)){
        _dateTimeIndex = i1;
        _dateTimeItem.isSelected = true;
      } else {
        _dateTimeItem.isSelected = false;
      }
      _dateTimeList.add(_dateTimeItem);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_format.format(_dateTimeSelected.toLocal())),
      ),
      body: ListView.builder(
        itemCount: _dateTimeList.length,
        itemExtent: 50,
        itemBuilder: (BuildContext context, int index) {
          return GestureDetector(
            onTap: (){
              if (_dateTimeList[index].canSelect){
                print('index = $index, '+_format.format(_dateTimeList[index].dateTime));
                setState(() {
                  _dateTimeList[_dateTimeIndex].isSelected = false;
                  _dateTimeIndex = index;
                  _dateTimeList[_dateTimeIndex].isSelected = true;
                  _dateTimeSelected = _dateTimeList[_dateTimeIndex].dateTime;
                });
              } else {
                print("index = $index, can't select");
              }
            },
            child: Container(
              color: _dateTimeList[index].isSelected?Colors.blueAccent:_dateTimeList[index].canSelect?Colors.transparent:Colors.grey.shade200,
              foregroundDecoration: BoxDecoration(
                border: Border(
                  bottom: BorderSide(
                    color: Theme.of(context).dividerColor,
                  ),
                ),
              ),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Text(
                    _format.format(_dateTimeList[index].dateTime),
//                    _format.format(_dateTimeList[index].dateTime.toUtc()),
//                        + '   Zone: ' +
//                        _dateTimeList[index].dateTime.toLocal().timeZoneOffset.inHours.toString(),
                    style: TextStyle(color: _dateTimeList[index].isSelected?Colors.white:_dateTimeList[index].canSelect?Colors.black:Colors.grey.shade500),
                    textAlign: TextAlign.center,
                  ),
                ],
              ),
            ),
          );
        },
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: _incrementCounter,
        tooltip: 'Increment',
        child: Icon(Icons.add),
      ),
    );
  }
}
